package si.url.rpr.activity;

import si.url.rpr.R;
import si.url.rpr.generator.GeneratorBuilder;
import si.url.rpr.snippet.Utils;
import si.url.rpr.tester.Tester;
import si.url.rpr.tester.WorkerPool;
import android.app.Activity;
import android.content.Context;
import android.net.DhcpInfo;
import android.net.wifi.WifiManager;
import android.os.Bundle;
import android.view.View;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.EditText;
import android.widget.Spinner;
import android.widget.TextView;
import android.widget.Toast;

public class BruteForce extends Activity {

	private boolean isRunning = false;

	private WorkerPool workerPool;

	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.brute_force);

		WifiManager wifi = (WifiManager) getSystemService(Context.WIFI_SERVICE);
		DhcpInfo d = wifi.getDhcpInfo();
		String gateway;
		if (d.gateway != 0) {
			gateway = Utils.intToIp(d.gateway);
		} else {
			gateway = "192.168.0.1";
		}

		final EditText url = (EditText) findViewById(R.id.url);
		url.setText("http://" + gateway);

		EditText charset = (EditText) findViewById(R.id.charset);
		charset.setText("abcdefghijklmnopqrstuvwxyz");

		Spinner threads = (Spinner) findViewById(R.id.threads);
		String[] threadsArray = { "1", "2", "3", "4" };
		ArrayAdapter<String> adapter = new ArrayAdapter<String>(this, android.R.layout.simple_spinner_item, threadsArray);
		adapter.setDropDownViewResource(android.R.layout.simple_dropdown_item_1line);
		threads.setAdapter(adapter);
		threads.setSelection(1);

		Button startstop = (Button) findViewById(R.id.startstop);
		startstop.setOnClickListener(new View.OnClickListener() {
			@Override
			public void onClick(View v) {
				if (isRunning) {
					stop();
					Toast.makeText(BruteForce.this, "Stopping threads", Toast.LENGTH_LONG).show();
				} else {

					new Thread(new Runnable() {
						@Override
						public void run() {
							Tester.testAuth(url.getText().toString(), new Tester.Callback2() {
								@Override
								public void onSuccess() {
									runOnUiThread(new Runnable() {
										@Override
										public void run() {
											start();
											Toast.makeText(BruteForce.this, "Starting threads", Toast.LENGTH_LONG).show();
										}
									});
								}

								@Override
								public void onFailure() {
									runOnUiThread(new Runnable() {
										@Override
										public void run() {
											Toast.makeText(BruteForce.this, "URL does not support HTTP AUTH", Toast.LENGTH_SHORT).show();
										}
									});
								}
							});
						}
					}).start();

				}
			}
		});
	}

	private void start() {

		isRunning = true;
		Button startstop = (Button) findViewById(R.id.startstop);
		startstop.setText("Stop");

		EditText etUsername = (EditText) findViewById(R.id.username);
		EditText etPasswordStart = (EditText) findViewById(R.id.passwordStart);
		EditText etUrl = (EditText) findViewById(R.id.url);
		EditText etCharset = (EditText) findViewById(R.id.charset);
		Spinner sThreads = (Spinner) findViewById(R.id.threads);

		String username = etUsername.getText().toString();
		String passwordStart = etPasswordStart.getText().toString();
		if (passwordStart.length() == 0)
			passwordStart = "a";
		final String url = etUrl.getText().toString();
		char[] charset = etCharset.getText().toString().toCharArray();
		int threads = Integer.parseInt((String) sThreads.getSelectedItem());

		GeneratorBuilder generatorBuilder = new GeneratorBuilder();
		generatorBuilder.setThreads(threads);
		generatorBuilder.setCharacterSet(charset);
		generatorBuilder.setPasswordStart(passwordStart);

		workerPool = new WorkerPool(generatorBuilder, url, username, threads, new WorkerPool.Callback() {
			@Override
			public void onFound(final String password) {
				final TextView passwordFound = (TextView) findViewById(R.id.passwordFound);
				runOnUiThread(new Runnable() {
					@Override
					public void run() {
						passwordFound.setText("Password found: " + password);
						stop();
					}
				});
			}

			@Override
			public void onFinish() {
				runOnUiThread(new Runnable() {
					@Override
					public void run() {
						stop();
					}
				});
			}
		});

		final TextView speed = (TextView) findViewById(R.id.speed);
		final TextView password = (TextView) findViewById(R.id.password);

		new Thread(new Runnable() {

			@Override
			public void run() {
				while (isRunning) {
					float s = workerPool.getThreadsSpeed();
					String current = workerPool.getCurrentPassword();
					updateStatus(s, current);

					try {
						Thread.sleep(5000);
					} catch (Exception e) {
					}
				}

				final String current = workerPool.getCurrentPassword();
				final EditText edPasswordStart = (EditText) findViewById(R.id.passwordStart);
				updateStatus(0, "");
				runOnUiThread(new Runnable() {
					@Override
					public void run() {
						edPasswordStart.setText(current);
					}
				});

			}

			private void updateStatus(final float s, final String current) {
				runOnUiThread(new Runnable() {
					@Override
					public void run() {
						speed.setText("Speed: " + String.format("%.4g", s) + " p/s");
						password.setText("Current password: " + current);
					}
				});

			}
		}).start();
	}

	private void stop() {
		Button startstop = (Button) findViewById(R.id.startstop);
		isRunning = false;
		startstop.setText("Start");
		workerPool.stop();
	}
}
