package si.url.rpr.activity;

import java.io.File;

import si.url.rpr.R;
import si.url.rpr.generator.GeneratorBuilder;
import si.url.rpr.snippet.SelectFile;
import si.url.rpr.snippet.Utils;
import si.url.rpr.tester.Tester;
import si.url.rpr.tester.WorkerPool;
import android.app.Activity;
import android.content.Context;
import android.net.DhcpInfo;
import android.net.wifi.WifiManager;
import android.os.Bundle;
import android.view.View;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.EditText;
import android.widget.Spinner;
import android.widget.TextView;
import android.widget.Toast;

public class Dictionary extends Activity {

	private boolean isRunning = false;
	private File dictionary = null;

	private WorkerPool workerPool;

	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.dictionary);

		WifiManager wifi = (WifiManager) getSystemService(Context.WIFI_SERVICE);
		DhcpInfo d = wifi.getDhcpInfo();
		String gateway;
		if (d.gateway != 0) {
			gateway = Utils.intToIp(d.gateway);
		} else {
			gateway = "192.168.0.1";
		}

		final EditText url = (EditText) findViewById(R.id.url);
		url.setText("http://" + gateway);

		Spinner threads = (Spinner) findViewById(R.id.threads);
		String[] threadsArray = { "1", "2", "3", "4" };
		ArrayAdapter<String> adapter = new ArrayAdapter<String>(this, android.R.layout.simple_spinner_item, threadsArray);
		adapter.setDropDownViewResource(android.R.layout.simple_dropdown_item_1line);
		threads.setAdapter(adapter);
		threads.setSelection(1);

		final Button dict = (Button) findViewById(R.id.selectDict);
		dict.setOnClickListener(new View.OnClickListener() {

			@Override
			public void onClick(View v) {
				SelectFile.selectDialog(Dictionary.this, new SelectFile.Callback() {
					@Override
					public void onSelect(String path) {
						dictionary = new File(path);
						dict.setText(dictionary.getName());
					}
				});
			}
		});

		Button startstop = (Button) findViewById(R.id.startstop);
		startstop.setOnClickListener(new View.OnClickListener() {
			@Override
			public void onClick(View v) {
				if (isRunning) {
					stop();
					Toast.makeText(Dictionary.this, "Stopping threads", Toast.LENGTH_LONG).show();
				} else {

					new Thread(new Runnable() {
						@Override
						public void run() {
							Tester.testAuth(url.getText().toString(), new Tester.Callback2() {
								@Override
								public void onSuccess() {
									runOnUiThread(new Runnable() {
										@Override
										public void run() {
											start();
											Toast.makeText(Dictionary.this, "Starting threads", Toast.LENGTH_LONG).show();
										}
									});
								}

								@Override
								public void onFailure() {
									runOnUiThread(new Runnable() {
										@Override
										public void run() {
											Toast.makeText(Dictionary.this, "URL does not support HTTP AUTH", Toast.LENGTH_SHORT).show();
										}
									});
								}
							});
						}
					}).start();

				}
			}
		});
	}

	private void start() {

		if (dictionary == null || !dictionary.exists()) {
			Toast.makeText(this, "Dictionary does not exist", Toast.LENGTH_LONG).show();
			return;
		}

		isRunning = true;
		Button startstop = (Button) findViewById(R.id.startstop);
		startstop.setText("Stop");

		EditText etUsername = (EditText) findViewById(R.id.username);
		EditText etUrl = (EditText) findViewById(R.id.url);
		Spinner sThreads = (Spinner) findViewById(R.id.threads);

		String username = etUsername.getText().toString();
		final String url = etUrl.getText().toString();
		int threads = Integer.parseInt((String) sThreads.getSelectedItem());

		GeneratorBuilder generatorBuilder = new GeneratorBuilder();
		generatorBuilder.setThreads(threads);
		generatorBuilder.setDictionary(dictionary);

		workerPool = new WorkerPool(generatorBuilder, url, username, threads, new WorkerPool.Callback() {
			@Override
			public void onFound(final String password) {
				final TextView passwordFound = (TextView) findViewById(R.id.passwordFound);
				runOnUiThread(new Runnable() {
					@Override
					public void run() {
						passwordFound.setText("Password found: " + password);
						stop();
					}
				});
			}

			@Override
			public void onFinish() {
				runOnUiThread(new Runnable() {
					@Override
					public void run() {
						stop();
					}
				});
			}
		});

		final TextView speed = (TextView) findViewById(R.id.speed);
		final TextView password = (TextView) findViewById(R.id.password);
		final TextView progress = (TextView) findViewById(R.id.progress);

		new Thread(new Runnable() {

			@Override
			public void run() {
				float percentage = 0;
				while (isRunning) {
					float speed = workerPool.getThreadsSpeed();
					percentage = workerPool.getThreadsPercentage();
					String current = workerPool.getCurrentPassword();
					updateStatus(speed, percentage, current);

					try {
						Thread.sleep(5000);
					} catch (Exception e) {
					}
				}

				percentage = workerPool.getThreadsPercentage();
				updateStatus(0, percentage, "");

			}

			private void updateStatus(final float s, final float p, final String current) {
				runOnUiThread(new Runnable() {
					@Override
					public void run() {
						speed.setText("Speed: " + String.format("%.4g", s) + " p/s");
						password.setText("Current password: " + current);
						progress.setText("Progress: " + String.format("%.4g", p) + " %");
					}
				});

			}
		}).start();
	}

	private void stop() {
		isRunning = false;
		Button startstop = (Button) findViewById(R.id.startstop);
		startstop.setText("Start");
		workerPool.stop();
	}
}
