package si.url.rpr.snippet;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;

import si.url.rpr.R;
import android.app.Activity;
import android.app.AlertDialog;
import android.app.Dialog;
import android.content.DialogInterface;
import android.os.Environment;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.ArrayAdapter;
import android.widget.ImageView;
import android.widget.ListView;
import android.widget.TextView;

public class SelectFile {

	// root for showing directories
	private static String locationRoot = Environment.getExternalStorageDirectory().toString() + "/";

	// adapter for displaying directories
	private static IconicAdapter adapter;
	private static ArrayList<File> adapterFiles = new ArrayList<File>();

	private static Dialog dialog;

	public static void selectDialog(Activity activity, final Callback cb) {

		AlertDialog.Builder dialogBuilder = new AlertDialog.Builder(activity);
		dialogBuilder.setTitle("Choose directory");

		LayoutInflater inflater = (LayoutInflater) activity.getSystemService(Activity.LAYOUT_INFLATER_SERVICE);
		View layout = inflater.inflate(R.layout.select_file, null);
		dialogBuilder.setView(layout);

		ListView list = (ListView) layout.findViewById(R.id.list);
		adapter = new IconicAdapter(activity, adapterFiles);
		list.setAdapter(adapter);
		list.setOnItemClickListener(new OnItemClickListener() {
			@Override
			public void onItemClick(AdapterView<?> arg0, View arg1, int position, long arg3) {
				// on click we load new directory structure
				File file = adapterFiles.get(position);

				if (file.isDirectory() && file.canRead()) {
					try {
						refreshList(file.getCanonicalPath());
					} catch (IOException e) {
						Utils.log(e);
					}
				} else if (file.isFile() && file.canRead()) {
					cb.onSelect(file.getAbsolutePath());
					dialog.dismiss();
				}
			}

		});
		dialogBuilder.setNegativeButton("Cancel", new DialogInterface.OnClickListener() {
			public void onClick(DialogInterface dialog, int whichButton) {
			}
		});
		refreshList(locationRoot);
		dialog = dialogBuilder.create();
		dialog.show();
	}

	private static void refreshList(String location) {
		adapterFiles.clear();
		adapterFiles.addAll(getDir(location));
		adapter.notifyDataSetChanged();
	}

	private static ArrayList<File> getDir(String location) {

		ArrayList<File> al = new ArrayList<File>();

		File dir = new File(location);
		File[] files = dir.listFiles();

		if (location.compareTo(locationRoot) != 0) {
			try {
				File back = new File(dir.getCanonicalPath(), "..");
				al.add(back);
			} catch (IOException e) {
				Utils.log(e);
			}
		}
		for (File f : files) {
			al.add(f);
		}

		return al;

	}

	public static interface Callback {
		public void onSelect(String path);
	}

	private static class IconicAdapter extends ArrayAdapter<String> {

		Activity activity;
		ArrayList<File> files;

		public IconicAdapter(Activity activity, ArrayList<File> files) {
			super(activity, R.layout.select_file_row, R.id.import_row_text);
			this.activity = activity;
			this.files = files;
		}

		public View getView(int position, View convertView, ViewGroup parent) {

			File file = files.get(position);
			View row = convertView;
			if (row == null) {
				LayoutInflater inflater = activity.getLayoutInflater();
				row = inflater.inflate(R.layout.select_file_row, parent, false);
			}

			((TextView) row.findViewById(R.id.import_row_text)).setText(file.getName());

			ImageView img = (ImageView) row.findViewById(R.id.import_row_fld_image);
			if (file.isFile()) {
				img.setImageResource(R.drawable.file);
			} else {
				img.setImageResource(R.drawable.folder);
			}

			return row;

		}

		@Override
		public int getCount() {
			return files.size();
		}

		@Override
		public String getItem(int position) {
			return files.get(position).getName();
		}

	}

}