package si.url.rpr.tester;

import si.url.rpr.generator.Generator;
import si.url.rpr.generator.GeneratorBuilder;

public class WorkerPool {

	private int threads = 1;
	private Worker[] pool;

	public WorkerPool(GeneratorBuilder generatorBuilder, String url, String username, int threads, final Callback cb) {
		this.threads = threads;
		pool = new Worker[threads];
		for (int i = 0; i < threads; i++) {
			Generator generator = generatorBuilder.build(i);
			pool[i] = new Worker(generator, url, username, threads, new Worker.Callback() {
				@Override
				public void onFound(String password) {
					stop();
					cb.onFound(password);
				}

				@Override
				public void onFinish() {
					for (int i = 0; i < pool.length; i++) {
						if (pool[i].isRunning())
							return;
					}
					cb.onFinish();
				}
			});
			pool[i].start();
		}
	}

	public float getThreadsSpeed() {
		float speed = 0;
		for (int i = 0; i < threads; i++) {
			speed += pool[i].getSpeed();
		}
		return speed;
	}

	public long getThreadsCount() {
		long count = 0;
		for (int i = 0; i < threads; i++) {
			count += pool[i].getCount();
		}
		return count;
	}

	public float getThreadsPercentage() {
		long count = getThreadsCount();
		long dictLength = pool[0].getGenerator().getLength();
		float percentage = (count / (float) dictLength) * 100;
		if (percentage > 100)
			percentage = 100;
		return percentage;
	}

	public String getCurrentPassword() {
		return pool[0].getPassword();
	}

	public void stop() {
		for (int i = 0; i < threads; i++) {
			pool[i].end();
		}
	}

	public interface Callback {
		public void onFound(String password);

		public void onFinish();
	}

}
